#version 430 core

#define PI 3.14159265359

// Uniforms
uniform vec2 u_Resolution;

// Outputs
out vec4 finalColor;

float insideCircle(float radius, float dist) 
{
    float smoothness = 0.01;
	return smoothstep(radius, radius - smoothness, dist);
}

float deg2Rad(float deg)
{
	return deg / 360.0 * 2.0 * PI;    
}

mat3 trans2D(vec2 trans) 
{
	return mat3(1.0, 0.0, 0.0,
               	0.0, 1.0, 0.0,
                trans.x, trans.y, 1.0);    
}

mat2 rotate2D(float angle)
{
	return mat2 (cos(angle), -sin(angle),
                 sin(angle), cos(angle));    
}

float insideRectangle(vec2 pos, vec2 rectPos, vec2 rectDim)
{
    float smoothness = 0.01;
	return smoothstep(rectPos.x - rectDim.x, rectPos.x - rectDim.x + smoothness, pos.x) * 
           (1.0 - smoothstep(rectPos.x + rectDim.x, rectPos.x + rectDim.x + smoothness, pos.x)) *
           smoothstep(rectPos.y - rectDim.y, rectPos.y - rectDim.y + smoothness, pos.y) * 
           (1.0 - smoothstep(rectPos.y + rectDim.y, rectPos.y + rectDim.y + smoothness, pos.y));    
}

void main() 
{
    vec2 pos = gl_FragCoord.xy / u_Resolution;
    pos = pos * 2.0 - vec2(1.0);
    //pos = rotate2D(u_time * 0.5) * pos;
    
    float dist = sqrt(pos.x * pos.x + pos.y * pos.y);
    float inside = insideCircle(1.0, dist);
    
    // Subtract inner circle
    inside *= (1.0 - insideCircle(0.75, dist));
    inside = max(insideCircle(0.05, dist), inside);
    inside = max(insideRectangle(rotate2D(PI/2.0 * 0.) * pos, vec2(-0.55, 0.0), vec2(0.3, 0.10)), inside);
    inside = max(insideRectangle(rotate2D(PI/2.0 * 1.) * pos, vec2(-0.55, 0.0), vec2(0.3, 0.10)), inside);
    inside = max(insideRectangle(rotate2D(PI/2.0 * 2.) * pos, vec2(-0.55, 0.0), vec2(0.3, 0.10)), inside);
    inside = max(insideRectangle(rotate2D(PI/2.0 * 3.) * pos, vec2(-0.55, 0.0), vec2(0.3, 0.10)), inside);
    
    finalColor = vec4(vec3(0.970,0.010,0.010), inside);
}