#version 430 core

// Outputs
out vec4 o_Color;

// Uniforms
uniform sampler2D u_DepthTexture;
uniform sampler2D u_NormalTexture;

// Globals
mat3 sx = mat3( 
    1.0, 2.0, 1.0, 
    0.0, 0.0, 0.0, 
    -1.0, -2.0, -1.0 
);
mat3 sy = mat3( 
    1.0, 0.0, -1.0, 
    2.0, 0.0, -2.0, 
    1.0, 0.0, -1.0 
);

void main()
{
	// Calc sobel on normals
    vec3 normal = texelFetch(u_NormalTexture, ivec2(gl_FragCoord), 0 ).xyz * 2.0f - 1.0f; //move normal back to [-1, 0] domain
    vec3 I[3];
    for (int i=0; i<3; i++) {
        vec3 norm1 = texelFetch(u_NormalTexture, ivec2(gl_FragCoord) + ivec2(i-1,-1), 0 ).rgb * 2.0f - 1.0f;
        vec3 norm2 =  texelFetch(u_NormalTexture, ivec2(gl_FragCoord) + ivec2(i-1,0), 0 ).rgb * 2.0f - 1.0f;
        vec3 norm3 = texelFetch(u_NormalTexture, ivec2(gl_FragCoord) + ivec2(i-1,1), 0 ).rgb * 2.0f - 1.0f;
        float sampleValLeft  = dot(normal, norm1);
        float sampleValMiddle  = dot(normal, norm2);
        float sampleValRight  = dot(normal, norm3);
        I[i] = vec3(sampleValLeft, sampleValMiddle, sampleValRight);
    }

	// Normal gradients
    float gx = dot(sx[0], I[0]) + dot(sx[1], I[1]) + dot(sx[2], I[2]); 
    float gy = dot(sy[0], I[0]) + dot(sy[1], I[1]) + dot(sy[2], I[2]);
	float g = length(vec2(gx, gy));

	// Calc sobel on depths
	float dgx = 0.0f;
	float dgy = 0.0f;
	for (int x = 0; x < 3; x++) {
		for (int y = 0; y < 3; y++) {
			float depthVal = texelFetch(u_DepthTexture, ivec2(gl_FragCoord) + ivec2 (x-1, y-1), 0).r;
			dgx += depthVal * sx[y][x];
			dgy += depthVal * sy[y][x];
		}
	}

	float dg = length(vec2(dgx, dgy)) * 100;
	// Threshhold 
	dg = dg > 1.0 ? 1.0 : 0.0;
	g = g > 1.5 ? 1.0 : 0.0;
	float final = min (dg + g, 1.0);

    // Write result
	o_Color = vec4(final, 0.0, 0.0, 1.0); // TODO: Use depth texture
}