#version 430 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;

layout(location = 2) uniform vec3 color;
layout(location = 3) uniform float ka;
layout(location = 4) uniform float kd;
layout(location = 5) uniform float ks;
layout(location = 6) uniform float alpha;

struct DirLight {
	vec3 color;
	vec3 dir;
};

struct PointLight {
	vec3 color;
	vec3 pos;
    vec3 attenuation;
};

struct SpotLight {
	vec3 dir;
	vec3 pos;
	vec3 color;
	float innerCosAngle;
	float outerCosAngle;
};


out VertexData {
	vec3 color;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform vec3 camPos;

uniform int dirLightLength;
uniform DirLight dirLight[10];
uniform int pointLightLength;
uniform PointLight pointLight[10];
uniform int spotLightLength;
uniform SpotLight spotLight[1];


void main() {
    vec3 world_position=vec3(modelMatrix*vec4(position,1));
	gl_Position = viewProjMatrix * vec4(world_position, 1);
	vec3 ambient=ka*vec3(1,1,1);
	vec3 toEye=normalize(world_position-camPos);

	vec3 specular=vec3(0.0f);
	vec3 diffuse=vec3(0.0f);

	for(int i=0; i<dirLightLength; i++){
		diffuse+=kd*max(0,dot(-normalize(dirLight[i].dir), normal))*dirLight[i].color;
		vec3 ref=2*dot(normalize(dirLight[i].dir),normal)*normal-normalize(dirLight[i].dir);//reflect(dirLight.dir, normal);
		specular+=ks*pow(max(dot(ref,toEye),0),alpha)*dirLight[i].color;
	}
	for(int i=0; i<pointLightLength;i++){
		vec3 pointLightDir=world_position-pointLight[i].pos;
		float dist=length(pointLightDir);
		float fatt=1/(dot(pointLight[i].attenuation,vec3(1,dist,dist*dist)));
		pointLightDir=normalize(pointLightDir);

		diffuse+=(kd*max(0,dot(-pointLightDir,normal))*pointLight[i].color)*fatt;
		vec3 ref=2*dot(pointLightDir,normal)*normal-pointLightDir;
		specular+=(ks*pow(max(dot(ref,toEye),0),alpha)*pointLight[i].color)*fatt;
	}
	for(int i=0;i<spotLightLength;i++) {
		vec3 v1=normalize(spotLight[i].dir);
		vec3 v2=normalize(world_position-spotLight[i].pos);
		float cosAngle=dot(v1,v2);
		if(cosAngle>spotLight[i].outerCosAngle) {
			float attenuation = smoothstep(spotLight[i].outerCosAngle,spotLight[i].innerCosAngle,cosAngle);
			diffuse+=kd*max(0,dot(-v2, normal))*spotLight[i].color*attenuation;
			vec3 ref=2*dot(v2,normal)*normal-v2;
			specular+=ks*pow(max(dot(ref,toEye),0),alpha)*spotLight[i].color*attenuation;
		}
	}
	vert.color=(ambient+diffuse)*color+specular;
}